// ad10led1.c - measure input voltage from pot divider using ADC10
// ADC10 default references (VSS, VCC), internal clock, polling loop
// Home-made analog demo board for MSP430F20xx:
//   10k on P1.5, 20k variable on P1.6, junction to P1.1 with 10nF cap
//   LED active high on P2.6
// Default DCO, no crystal, no ACLK, power from JTAG
// J H Davies, 2007-08-30; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x2.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = BIT5;					// VCC to 10k fixed resistor
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
// VCC and VSS refs, sample for 16 cycs, int ref off, ADC on, no ints
	ADC10CTL0 = SREF_0 | ADC10SHT_2 | ADC10ON;
// Input channel 1, trigger using ADC10SC bit, no clock division,
//   internal ADC clock, single channel single conversions
	ADC10CTL1 = INCH_1 | SHS_0 | ADC10DIV_0 | ADC10SSEL_0 | CONSEQ_0;
	ADC10AE0 = BIT1;				// Enable analog input on channel 1
	ADC10CTL0_bit.ENC = 1;			// Enable conversions
	for (;;) {						// Loop forever taking measurements
		ADC10CTL0_bit.ADC10SC = 1;	// Trigger new conversion
		while (ADC10CTL1_bit.ADC10BUSY == 1) {
		}							// Loop while conversion takes place
		if (ADC10MEM >= BIT9) {		// Is output in top half of range?
			LED = 1;				// Yes: Turn LED on
		} else {
			LED = 0;				// No: Turn LED off
		}
	}
}
