// ad10pwm3.c - PWM controlled by input voltage using ADC10
// ADC10 def refs (VSS, VCC), int clock, auto disabled between readings
// Repeated conversions initiated by OUT0 in toggle mode
// PWM updated by data transfer controller (DTC) so CPU not needed!
// Home-made analog demo board for MSP430F20xx:
//   10k on P1.5, 20k variable R on P1.6, junction to P1.1 with 10nF cap
//   LED active high on P2.6 driven from OUT1 in PWM
// Default DCO, no crystal, no ACLK, power from JTAG
// J H Davies, 2007-08-30; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x2.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	P2SEL = BIT6;					// OUT1 on P2.6; no crystal
	P2OUT = 0;						// Unused outputs driven low
	P2DIR = BIT6|BIT7;
	P1OUT = BIT5;					// VCC to 10k fixed resistor
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
// Timer_A for PWM at 125Hz on OUT1, SMCLK/8, Up mode
	TACCR0 = BITA - 1;				// Upper limit to match ADC10MEM
	TACCTL0 = OUTMOD_4;				// Toggle OUT0 to stimulate ADC10
	TACCR1 = BIT9;					// About 50% to start PWM
	TACCTL1 = OUTMOD_7;				// Reset/set for positive PWM
	TACTL = TASSEL_2 | ID_3 | MC_1 | TACLR;	// SMCLK/8, Up mode, clear
// ADC on, refs VCC VSS, sample 4 cycles, int ref off, no interrupts
	ADC10CTL0 = SREF_0 | ADC10SHT_0 | ADC10ON;
// Input channel 1, start on rising edge of OUT0, no clock division,
//   internal ADC clock, single channel repeated conversions
	ADC10CTL1 = INCH_1 | SHS_2 | ADC10DIV_0 | ADC10SSEL_0 | CONSEQ_2;
	ADC10AE0 = BIT1;				// Enable analog input on channel 1
// Data transfer controller: copy one result continuously to TACCR1
	ADC10DTC0 = ADC10CT;			// Continuous transfers, one block
	ADC10DTC1 = 1;					// Single target address in memory
	ADC10SA = (unsigned short) &TACCR1;	// (Starting) target address
	ADC10CTL0_bit.ENC = 1;			// Enable ADC10 conversions
	for (;;) {						// Loop forever taking measurements
		__low_power_mode_0();		// CPU no longer needed,
	}								//   not even for interrupts!
}
