// ad10tmp1.c - measure temperature in celsius using sensor in ADC10
// ADC10 and ref disabled between readings set by watchdog timer
// ADC10 internal 1.5V ref, internal clock/4, interrupt on finish
// Home-made analog demo board for MSP430F20xx, LED active high on P2.6
// Default DCO, no crystal, ACLK from VLO, power from JTAG
// J H Davies, 2007-08-30; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x2.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED

void main (void)
{
	uint32_t temperature;			// Converted value of temperature

	BCSCTL3 = LFXT1S_2;				// Select ACLK from VLO (no crystal)
// Watchdog as interval timer, clear, ACLK / 32768
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL;
	IE1_bit.WDTIE = 1;				// WDT interval timer interrupts
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = 0;						// All outputs low
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
// Vref+ = 1.5V and VSS refs, sample 64 cycs, slow, ref + ADC off, ints
	ADC10CTL0 = SREF_1 | ADC10SHT_3 | ADC10SR | ADC10IE;
// Input channel 10 (temperature), trigger using ADC10SC bit, binary,
//   internal osc/4, single channel single conversions
	ADC10CTL1 = INCH_10 | SHS_0 | ADC10DIV_3 | ADC10SSEL_0 | CONSEQ_0;
	for (;;) {						// Loop forever taking measurements
		__low_power_mode_3();		// Wait for WDT between measurements
		ADC10CTL0 |= (REFON|ADC10ON);	// Enable ADC10 and reference
		ADC10CTL0 |= (ENC|ADC10SC);	// Enable ADC10, start conversion
		__low_power_mode_3();		// Sleep during measurement
		ADC10CTL0_bit.ENC = 0;		// Disable further ADC10 conversions
		ADC10CTL0 &= ~(REFON|ADC10ON);	// Disable ADC10 and reference
		temperature = ADC10MEM;		// Raw converted value
		temperature *= 420;			// Scaling factor (Vref/temp coeff)
		temperature += 512;			// Allow for rounding in division
		temperature /= 1024;		// Divide by range of ADC10
		temperature -= 278;			// Subtract offset to give celsius
		if (temperature > 25) {		// Is temperature above 25 celsius?
			LED = 1;				// Yes: Turn LED on
		} else {
			LED = 0;				// No: Turn LED off
		}
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for watchdog after sleep: return to active
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
//----------------------------------------------------------------------
// Interrupt service routine for ADC10 after converson: return to active
//----------------------------------------------------------------------
#pragma vector = ADC10_VECTOR
__interrupt void ADC10_ISR (void)	// Acknowledged automatically
{
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
