// dacdma2.c - Convert stored wave repeatedly using DAC12 and DMA
// DMA triggered by DAC12_0, which is itself triggered by Timer_A.OUT1
// DAC12_0 output buffered by OA2 (gain = 2) to drive headphone
// TI Experimenter's Board with F4619, default clocks, LCD not used
// J H Davies, 2007-10-13; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils.h"				// TI exp board utility functions
// Add definition that seems to be missing from io430xG46x.h
__no_init volatile unsigned long DMA0SZ @ DMA0SZ_;

// Lookup table, sine wave, full cycle, bipolar, signed bytes (+/-127)
const int8_t wave[] = {	   0,   25,   49,   71,   90,  106,  117,  125,
						 127,  125,  117,  106,   90,   71,   49,   25,
						   0,  -25,  -49,  -71,  -90, -106, -117, -125,
						-127, -125, -117, -106,  -90,  -71,  -49,  -25};

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;			// Configure load caps (14pF)
	PortsInit();					// Initialize i/o ports
// Configure op-amp 2 as a buffer for the DAC12 output
    OA2CTL1 = OAFC_4 | OAFBR_2 | OARRIP;	// Noninv, gain = 2, no RRIP
// Noninverting input from DAC0, fast (high current), external output A5
    OA2CTL0 = OAP_2 | OAPM_3 | OAADC1;
	ADC12CTL0 = REFON;				// ADC12 1.5V ref on, needed for DAC
// Configure DAC12_0 for audio output to buffer (OA2); do not enable
// Internal Vref, 8-bit, TA1 trigger, gain = 1, max speed, 2s complement
	DAC12_0CTL = DAC12SREF_0 | DAC12RES | DAC12LSEL_2 | DAC12IR
					| DAC12AMP_7 | DAC12DF;	// DAC12 not yet enabled!
	DAC12_0CTL_bit.DAC12CALON = 1;	// Start calibration
	while (DAC12_0CTL_bit.DAC12CALON == 1) {	// (Should stop CPU)
	}								// Wait for calibration to complete
// Configure DMA chan 0 to copy wave[] table to DAC12_0; do not enable
	DMACTL0 = DMA0TSEL_5;			// Trigger channel 0 from DAC12_0IFG
	DMACTL1 = 0;					// DMA occurs immediately (default)
// Repeated single transfers, fixed dest, increment source, bytes
	DMA0CTL = DMADT_4|DMADSTINCR_0|DMASRCINCR_3|DMADSTBYTE|DMASRCBYTE;
	DMA0SA = (uintptr_t) wave;		// Source address: wave array
	DMA0DA = (uintptr_t) &DAC12_0DAT;	// Destination: DAC12 data
	DMA0SZ = sizeof(wave) / sizeof(int8_t);	// Number of transfers
// Configure Timer_A to produce square wave on OUT1 to trigger DAC12_0
	TACCR0 = 1;						// Period = 2 (highest frequency)
	TACCR1 = 1;						// To generate OUT1 for DAC12_0
	TACCTL1 = OUTMOD_7;				// Reset/set for active high PWM
// Everything is now configured: start the action!
	DMA0CTL_bit.DMAEN = 1;			// Enable DMA channel 0
	DAC12_0CTL_bit.DAC12ENC = 1;	// Enable DAC12 channel 0
    DAC12_0CTL_bit.DAC12IFG = 1;	// Stimulate DMA to write DAC12_0DAT
// Start Timer_A from ACLK, undivided, Up mode, clear
	TACTL = TASSEL_1 | ID_0 | MC_1 | TACLR;	// Start triggers to DAC12_0
	for (;;) {						// Enter LPM3, only ACLK needed
		__low_power_mode_3();		// (Interrupts enabled but not used)
	}								// CPU no longer needed
}
