// butled4.c - press button B1 to light LED1
// Responds to interrupts on input pin, LPM4 between interrupts
// Olimex 1121STK board, LED1 active low on P2.3,
//   button B1 active low on P2.1
// J H Davies, 2006-11-18; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	P2OUT_bit.P2OUT_3 = 1;			// Preload LED1 off (active low!)
	P2DIR_bit.P2DIR_3 = 1;			// Set pin with LED1 to output
	P2IE_bit.P2IE_1 = 1;			// Enable interrupts on edge
	P2IES_bit.P2IES_1 = 1;			// Sensitive to negative edge (H->L)
	do {
		P2IFG = 0;					// Clear any pending interrupts...
	} while (P2IFG != 0);			// ...until none remain
	for (;;) {						// Loop forever (should not need)
		__low_power_mode_4();		// LPM4 with int'pts, all clocks off
	}								//   (RAM retention mode)
}
//----------------------------------------------------------------------
// Interrupt service routine for port 2 inputs
// Only one bit is active so no need to check which
// Toggle LED, toggle edge sensitivity, clear any pending interrupts
// Device returns to low power mode automatically after ISR
//----------------------------------------------------------------------
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR (void)
{
	P2OUT_bit.P2OUT_3 ^= 1; 		// Toggle LED
	P2IES_bit.P2IES_1 ^= 1;			// Toggle edge sensitivity
	do {
		P2IFG = 0;					// Clear any pending interrupts...
	} while (P2IFG != 0);			// ...until none remain
}
