// timainC1.c - toggles LEDs with period of about 1s
// Processor enters low power mode 3 between ISRs (need ACLK for timer)
// Toggle LEDs in mainline loop paced by interrupts from timer A CCR0
//   in Up mode with period of about 0.5s
// Timer clock is 32KHz ACLK, no division, up mode, period 0x4000
// Olimex 1121STK, LED1,2 active low on P2.3,4
// J H Davies, 2006-10-11; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
//----------------------------------------------------------------------
// Pins for LEDs
#define	LED1	BIT3
#define	LED2	BIT4
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW|WDTHOLD;			// Stop watchdog timer
	P2OUT = ~LED1;					// Preload LED1 on, LED2 off
	P2DIR = LED1|LED2;				// Set pins with LED1,2 to output
	TACCR0 = 0x4000;				// Upper limit of count for TAR
	TACCTL0 = CCIE;					// Enable interrupts on Compare 0
	TACTL = MC_1|TASSEL_1|TACLR;	// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
	for (;;) {						// Loop forever
		__low_power_mode_3();		// Enter low power mode LPM3
// Wait here, pace loop until timer expires and ISR restores Active Mode
		P2OUT ^= LED1|LED2;			// Toggle LEDs
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Processor remains in Active Mode after ISR
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR (void)
{
	__low_power_mode_off_on_exit();		// Restore Active Mode on return
}
