// timintC1.c - toggles LEDs with period of about 1.0s
// Toggle LEDs in ISR using interrupts from timer A CCR0
//   in Up mode with period of about 0.5s
// Timer clock is SMCLK divided by 8, up mode, period 50000
// Olimex 1121STK, LED1,2 active low on P2.3,4
// J H Davies, 2006-10-11; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430x11x1.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
//----------------------------------------------------------------------
// Pins for LEDs
#define	LED1	BIT3
#define	LED2	BIT4
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW|WDTHOLD;				// Stop watchdog timer
	P2OUT = ~LED1;						// Preload LED1 on, LED2 off
	P2DIR = LED1|LED2;					// Set pins with LED1,2 to output
	TACCR0 = 49999;						// Upper limit of count for TAR
	TACCTL0 = CCIE;						// Enable interrupts on Compare 0
	TACTL = MC_1|ID_3|TASSEL_2|TACLR;	// Set up and start Timer A
// "Up to CCR0" mode, divide clock by 8, clock from SMCLK, clear timer
	__enable_interrupt();				// Enable interrupts (intrinsic)
	for (;;) {							// Loop forever doing nothing
	}									// Interrupts do the work!
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR (void)
{
		P2OUT ^= LED1|LED2;				// Toggle LEDs
}
