// freqgen1.c - generate a precise average 880Hz frequency on output
// Shortfall is accumulated and correction made when threshold exceeded
// timer clock =   1,   2,   4,   8,  16,  32 KHz (divide timer clock
// QUOTIENT    =   1,   2,   4,   9,  18,  37       and ACLK)
// REMAINDER   = 144, 288, 576, 272, 544, 208
// Timer_A, Continuous mode, Channel 1 output, clock = 1KHz to 32KHz
// Olimex 1121STK, LED1 on P2.3 = TA1, piezo between P2.0 and P2.5
// J H Davies, 2007-06-24; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x11x1.h>			// Specific device, newer format
#include <intrinsics.h>			// Intrinsic functions
#include <stdint.h>				// Integers of defined sizes
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
//	BCSCTL1 |= DIVA_1;			// Divide ACLK by 4 to 8KHz
// Channel 1 of timer, drives LED1 directly (P2.3 = TA1), active low
	TACCTL1 = OUTMOD_0 | OUT;	// Force output initially high (LED off)
	TACCR1 = 10;				// Initial dummy interval
	TACCTL1 = OUTMOD_4 | CCIE;	// Change to toggle mode with interrupts
	P1OUT = BIT0 | BIT1;		// Output, high for Freq pin and TXD
	P1DIR = BIT0 | BIT1;
	P2SEL = BIT3;				// Re-route P2.3 to Timer TA1
	P2OUT = BIT0 | BIT3 | BIT4;	// LEDs off (active low); prepare piezo
	P2DIR = BIT0 | BIT3 | BIT4 | BIT5;	// Piezo and LED outputs
// Start timer from ACLK, full speed, continuous mode, clear
	TACTL = TASSEL_1 | ID_0 | MC_2 | TACLR;
// Start timer from ACLK, divide by 8, continuous mode, clear
//	TACTL = TASSEL_1 | ID_3 | MC_2 | TACLR;
	for (;;) {					// Loop forever
		__low_power_mode_3();	// LPM3 between CCIFG1 interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for CCIFG1 (only source active for vector)
//----------------------------------------------------------------------
#define	FREQUENCY	880			// Desired frequency of events
// Generally: f_timer = QUOTIENT * FREQUENCY + REMAINDER
//   or QUOTIENT = f_timer / FREQUENCY; REMAINDER = f_timer % FREQUENCY
#define QUOTIENT	37			// values for 32KHz timer clock
#define REMAINDER	208
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// Flag NOT cleared automatically!
{
	static uint16_t shortfall = 0;	// Accumulated (summed) shortfall

	TACCTL1_bit.CCIFG = 0;			// Acknowledge interrupt
	P2OUT ^= (BIT0|BIT5);			// Toggle piezo sounder
	shortfall += REMAINDER;			// Update accumulated shortfall
	if (shortfall >= FREQUENCY) {	// Over threshold for extra count?
		TACCR1 += (QUOTIENT + 1);	// Yes: extra count in output period
		shortfall -= FREQUENCY;		// Subtrct correction from shortfall
	} else {
		TACCR1 += QUOTIENT;			// No: usual count
	}
}
