// react1.c - Reaction timer using Timer_A, display in ms
// Press S2, wait for LED4, press S1 as quickly as possible!
// Timer_A used for random delay and reaction time
// Actions take place in ISRs
// TI Experimenter's Board with F4619, default clocks
// J H Davies, 2007-06-11; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils.h"				// SBLCDA4 utility functions
#define	LED		P5OUT_bit.P5OUT_1	// Pin for LED
void PortsInit (void);				// Set up i/o ports
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;			// Configure load caps (14pF)
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
// Capture falling edge of CCI0A, synchronized, no ints yet
	TACCTL0 = CM_2 | CCIS_0 | SCS | CAP;
// Start timer: ACLK, no prescale, continuous mode, no ints yet, clear
	TACTL = TASSEL_1 | ID_0 | MC_2 | TACLR;
	P1IFG = 0;						// Clear any pending interrupts
	P1IE = BIT1;					// Enable interrupts on P1.1 = B2
	for (;;) {						// Loop forever with interrupts
		__low_power_mode_3();		// Only ACLK continues to run
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for port 1; need to clear flag
// Enable TAIFG interrupts to give 1-2s delay before reaction test
//----------------------------------------------------------------------
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)	// Flag NOT cleared automatically!
{
	P1IFG = 0;						// Acknowledge interrupt
	P1IE = 0;						// Disable further P1 interrupts
	TACTL_bit.TAMC = 0;				// Hold timer
	if (TAR > 0x8000) {				// Will delay be < 1s?
		TAR -= 0x8000;				// Yes: add 1s so 1s < delay < 2s
	}
	TACTL_bit.TAIFG = 0;			// Clear any pending interrupt
	TACTL |= MC_2 | TAIE;			// Restart timer with interrupts
}
//----------------------------------------------------------------------
// Interrupt service routine for TAIFG, called after random interval
// Check source to clear flag and "service" erroneous interrupts
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// Flag NOT cleared automatically!
{
	if (TAIV == TAIV_TAIFG) {		// Vector 10: TAIFG
		LED = 1;					// Turn on LED to start trial
		TACTL_bit.TAIE = 0;			// Disable further TAIFG interrupts
		TACCTL0_bit.CCIFG = 0;		// Clear any pending interrupt
		TACCTL0_bit.CCIE = 1;		// Enable TACCR0 capture interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCR0.CCIFG, called on capture
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)	// Flag cleared automatically
{
	uint32_t ReactionTime;			// Computed reaction time, 32 bits

	TACCTL0_bit.CCIE = 0;			// Disable further CCIFG0 interrupts
	ReactionTime = TACCR0;			// Starting time was 0 counts
	ReactionTime *= 1000;			// Convert seconds to ms
	ReactionTime >>= 15;			// Economical /= 32K (f_ACLK)
	DisplayUint ((uint16_t) ReactionTime);	// Truncate and display
	LED = 0;						// Turn off LED
	P1IFG = 0;						// Clear any pending interrupts
	P1IE = BIT1;					// Re-enable interrupts on P1.1 = B2
}
//----------------------------------------------------------------------
// Initialize ports for board: pins are outputs driven low by default
// Many of these will be overwritten by LCD initialization later
//----------------------------------------------------------------------
void PortsInit (void)
{
// Port 1: 0 = SW1, 1 = SW2, others used for Chipcon (not placed)
	P1OUT = 0;
	P1DIR = 0xFF & ~(BIT0|BIT1);		// P1.0,1 input, others output
	P1SEL = BIT0;						// P1.0 = S1 to Timer_A CCI0A
	P1IES = BIT1;						// Interrupts on falling edge
// Port 2: 1 = LED2, 2 = LED1, 3 = mic power, 4 = UCA0TXD, 5 = UCA0RXD
	P2OUT = 0;
	P2DIR = 0xFF & ~BIT5;				// P2.5 must be input
// Port 3: 0-3 = comms with F2013, 5 = buzzer (active low)
//	P3OUT = BIT1 | BIT2;				// F2013 might tie these high
//	P3DIR = 0xFF & ~BIT5;				// P3.5 must be high
	P3OUT = BIT1 | BIT2 | BIT5;			// F2013 might tie 1,2 high;
	P3DIR = 0xFF;						//   P3.5 high for buzzer off
// Port 4: 2-5 used for Chipcon (not placed)
	P4OUT = 0;
	P4DIR = 0xFF;
// Port 5: 1 = LED4, 2-4 = COM1-3 for LCD
	P5OUT = 0;
	P5DIR = 0xFF;
// Port 6: All except 6 used for opamp external components
	P6OUT = 0;
	P6DIR = 0xFF;
// Port 7: 4-7 = LCD segments
	P7OUT = 0;
	P7DIR = 0xFF;
// Port 8: LCD segments
	P8OUT = 0;
	P8DIR = 0xFF;
// Port 9: LCD segments
	P9OUT = 0;
	P9DIR = 0xFF;
// Port 10: 0-5 = LCD segments
	P10OUT = 0;
	P10DIR = 0xFF;
}
