// simppwm1.c - Simple, slow PWM using Timer_A in Up mode from ACLK
// Period = 1s, LED1 at duty cycle D = 1/2, LED2 at D = 1/4
// Olimex 1121STK, LED1 on P2.3 = TA1, LED2 on P2.4 = TA2, active low
// J H Davies, 2007-07-17; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x11x1.h>			// Specific device
#include <intrinsics.h>			// Intrinsic functions
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
// Configure ports 1 and 2; redirect P2.3,4 to Timer_A
	P1OUT = BIT0 | BIT1;		// Output high for Freq pin and TXD
	P1DIR = BIT0 | BIT1;
	P2SEL = BIT3 | BIT4;		// Re-route P2.3 to TA1, P2.4 to TA2
	P2DIR = BIT0 | BIT3 | BIT4 | BIT5;	// Piezo and LED outputs
// Set up Timer_A for PWM on active low LEDs, channels 1 and 2
	TACCR0 = 0x7FFF;			// 1s period from 32KHz timer clock
	TACCR1 = 0x4000;			// Duty cycle D = 1/2
	TACCR2 = 0x2000;			// Duty cycle D = 1/4
	TACCTL1 = OUTMOD_3;			// Set/reset mode for negative PWM
	TACCTL2 = OUTMOD_3;			// Set/reset mode for negative PWM
// Start timer from ACLK, no division, Up mode, clear, no interrupts
	TACTL = TASSEL_1 | ID_0 | MC_1 | TACLR;
	for (;;) {					// Loop forever
		__low_power_mode_3();	// Remain in LPM3, CPU not needed!
	}							//   (nor are interrupts)
}
