// tmrints.c - demonstrate shared interrupt vector for Timer_A
// Flash LED, set on TAIFG, clear on CCIFG1
// TI eZ430, F2013, LED on P1.0, ACLK from VLO, MCLK and SMCLK from DCO
// J H Davies, 2007-05-29; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x3.h>			// Header file for this device
#include <intrinsics.h>			// Intrinsic functions

void main( void )
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
	BCSCTL3 = LFXT1S_2;			// Select ACLK from VLO (no crystal)
//  Initialize ports
	P1OUT = 0;					// Preset LED off (and everything else)
	P1DIR = BIT0;				// Configure P1.0 for LED output
	P1REN = 0xFE;				// Activate pullups on P1[7:1]
	P2SEL = 0;					// Digital i/o rather than crystal
	P2REN = BIT6|BIT7;			// Activate pullups on P2[7:6]
// Set up timer channel 1
	TACCR1 = 0x2000;			// Test compare action
	TACCTL1 = CCIE;				// Interrupts on TACCR1 compare
// Start timer: SMCLK, prescale/8, continuous mode, interrupts, clear
	TACTL = TASSEL_2 | ID_3 | MC_2 | TACLR | TAIE;
	for (;;) {
		__low_power_mode_0();	// LPM0 with interrupts (need SMCLK)
	}							// Infinite loop
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCR1.CCIFG, called when TAR -> TACCR1
//   and for TAIFG, called when TAR -> 0; share vector
// Interrupt flag cleared automatically with access to TAIV
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// ISR for TACCR1 CCIFG and TAIFG
{
//	switch (TAIV) {				// Standard switch
	switch (__even_in_range(TAIV, 10)) {	// Faster intrinsic fn
	case 0:						// No interrupt pending
		break;					// No action
	case TAIV_CCIFG1:			// Vector 2: CCIFG1
		P1OUT_bit.P1OUT_0 = 0;	// End of duty cycle: Turn off LED
		break;
	case TAIV_TAIFG:			// Vector A: TAIFG, last value possible
		P1OUT_bit.P1OUT_0 = 1;	// Start of PWM cycle: Turn on LED
		break;
	default:					// Should not be possible
		for (;;) {				// Disaster! Loop here for ever
		}
	}
}
